LHR = select(2, ...)

local L = LHR.L

local parent = ...
LHR.version = GetAddOnMetadata(parent, "Version")

-- Initialize addon - Shouldn't be call more than once
function LHR:init()

    self:LoadDefaults()

    self.db = LibStub:GetLibrary("AceDB-3.0"):New("LoathebHeilerRotationDb", self.defaults, true)
    self.db.RegisterCallback(self, "OnProfileChanged", "ProfilesChanged")
    self.db.RegisterCallback(self, "OnProfileCopied", "ProfilesChanged")
    self.db.RegisterCallback(self, "OnProfileReset", "ProfilesChanged")

    self:CreateConfig()

    LHR.healerTable = {}
    LHR.rotationTables = { rotation = {}, backup = {} }

    LHR.raidInitialized = false
    LHR.testMode = false

    LHR:initGui()
    LHR:updateRaidStatus()
    LHR:applySettings()
    LHR:updateDragAndDrop()

    LHR:initComms()

    LHR:printMessage(L['LOADED_MESSAGE'])
end

-- Apply setting on profile change
function LHR:ProfilesChanged()
	self.db:RegisterDefaults(self.defaults)
    self:applySettings()
end

-- Apply settings
function LHR:applySettings()

    LHR.mainFrame:ClearAllPoints()

    local config = LHR.db.profile
    if config.point then
        LHR.mainFrame:SetPoint(config.point, UIParent, 'BOTTOMLEFT', config.x, config.y)
    else
        LHR.mainFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    end

    LHR:updateDisplay()

    LHR.mainFrame:EnableMouse(not LHR.db.profile.lock)
    LHR.mainFrame:SetMovable(not LHR.db.profile.lock)
end

-- Print wrapper, just in case
function LHR:printMessage(msg)
    print(msg)
end

-- Print message with colored prefix
function LHR:printPrefixedMessage(msg)
    LHR:printMessage(LHR:colorText(LHR.constants.printPrefix) .. msg)
end

-- Send a heal announce message to a given channel
function LHR:sendAnnounceMessage(message, targetName)
    if LHR.db.profile.enableAnnounces then
        LHR:sendMessage(
            message,
            targetName,
            LHR.db.profile.channelType,
            LHR.db.profile.targetChannel
        )
    end
end

-- Send a rotation broadcast message
function LHR:sendRotationSetupBroadcastMessage(message)
    if LHR.db.profile.enableAnnounces then
        LHR:sendMessage(
            message,
            nil,
            LHR.db.profile.rotationReportChannelType,
            LHR.db.profile.setupBroadcastTargetChannel
        )
    end
end

-- Send a message to a given channel
function LHR:sendMessage(message, targetName, channelType, targetChannel)
    local channelNumber
    if channelType == "CHANNEL" then
        channelNumber = GetChannelName(targetChannel)
    end
    SendChatMessage(string.format(message, targetName), channelType, nil, channelNumber or targetChannel)
end

SLASH_LHR1 = "/lhr"
SLASH_LHR2 = "/loatheb"
SlashCmdList["LHR"] = function(msg)
    local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")

    if (cmd == 'ui') then
        LHR:toggleDisplay()
    elseif (cmd == 'backup') then
        LHR:alertBackup(LHR.db.profile.unableToHealMessage)
    elseif (cmd == 'report') then
        LHR:printRotationSetup()
	elseif (cmd == 'test') then
		LHR:ToggleTestMode(false)
    elseif (cmd == 'settings') then
        LHR:openSettings()
    else
        LHR:printHelp()
    end
end

function LHR:toggleDisplay()
    if (LHR.mainFrame:IsShown()) then
        LHR.mainFrame:Hide()
        LHR:printMessage(L['HEAL_WINDOW_HIDDEN'])
    else
        LHR.mainFrame:Show()
    end
end

-- Open ace settings
function LHR:openSettings()
    local AceConfigDialog = LibStub("AceConfigDialog-3.0")
    AceConfigDialog:Open("LoathebHeilerRotation")
end

-- Sends rotation setup to raid channel
function LHR:printRotationSetup()

    if (IsInRaid()) then
        LHR:sendRotationSetupBroadcastMessage('--- ' .. LHR.constants.printPrefix .. L['BROADCAST_HEADER_TEXT'] .. ' ---', channel)

        if (LHR.db.profile.useMultilineRotationReport) then
            LHR:printMultilineRotation(LHR.rotationTables.rotation)
        else
            LHR:sendRotationSetupBroadcastMessage(
                LHR:buildGroupMessage(L['BROADCAST_ROTATION_PREFIX'] .. ' : ', LHR.rotationTables.rotation)
            )
        end

        if (#LHR.rotationTables.backup > 0) then
            LHR:sendRotationSetupBroadcastMessage(
                LHR:buildGroupMessage(L['BROADCAST_BACKUP_PREFIX'] .. ' : ', LHR.rotationTables.backup)
            )
        end
    end
end

-- Print the main rotation on multiple lines
function LHR:printMultilineRotation(rotationTable, channel)
    local position = 1;
    for key, healer in pairs(rotationTable) do
        LHR:sendRotationSetupBroadcastMessage(tostring(position) .. ' - ' .. healer.name)
        position = position + 1;
    end
end

-- Serialize healers names of a given rotation group
function LHR:buildGroupMessage(prefix, rotationTable)
    local healers = {}

    for key, healer in pairs(rotationTable) do
        table.insert(healers, healer.name)
    end

    return prefix .. table.concat(healers, ', ')
end

-- Print command options to chat
function LHR:printHelp()
    local spacing = '   '
    LHR:printMessage(LHR:colorText('/lhr') .. ' console Optionen :')
    LHR:printMessage(spacing .. LHR:colorText('ui') .. ' : Zeigt/Versteckt Fenster')
    LHR:printMessage(spacing .. LHR:colorText('test') .. ' : Startet den Testmodus')
    LHR:printMessage(spacing .. LHR:colorText('settings') .. ' : Öffnet LHR Einstellungen')
    LHR:printMessage(spacing .. LHR:colorText('report') .. ' : Heiler Rotation ansagen')
    LHR:printMessage(spacing .. LHR:colorText('backup') .. ' : Flüstert den Backup an')
end

-- Adds color to given text
function LHR:colorText(text)
    return '|cffffbf00' .. text .. '|r'
end

-- Toggle arcane shot testing mode
function LHR:ToggleTestMode(disable)

    if (not disable and not LHR.testMode) then
        LHR:printPrefixedMessage(L['TESTING_ENABLED'])
        LHR.testMode = true

        -- Disable testing after 10 minutes
        C_Timer.After(600, function()
            LHR:ToggleTestMode(true)
        end)
    else
        LHR.testMode = false
        LHR:printPrefixedMessage(L['TESTING_DISABLED'])
    end
end
